from typing import Generic, Optional, TypeVar

from pydantic import BaseModel

T = TypeVar("T")


class NoSpecPayload(BaseModel):
    tenant_name: str
    app_id: str
    job_id: str


F = TypeVar("F", bound=NoSpecPayload)


class Payload(NoSpecPayload, Generic[T]):
    # token_id: str # TODO: add token_id to enforce tenant isolation
    spec: T


class PartPayload(BaseModel, Generic[F]):
    inputs: F


class PayloadReturn(BaseModel, Generic[T]):
    payload: Optional[T]
    message: Optional[str]



from enum import Enum
from typing import List, Optional

from pydantic import BaseModel, ConfigDict


class ProvisionerComponent(BaseModel):
    type: str
    name: str

    model_config = ConfigDict(
        extra="allow",
    )


class Step(BaseModel):
    name: str
    type: str
    queue: str
    method: str


class Notifier(BaseModel):
    complete: Optional[List[Step]]
    failure: Optional[List[Step]]


class Dag(BaseModel):
    steps: List[Step]
    notifiers: Notifier
    provisioner: List[ProvisionerComponent]


class ProvisionerType(Enum):
    DEPLOYMENT = "deployment"
    CRON = "cron"


class Resource(BaseModel):
    cpu: str
    memory: str


class Resources(BaseModel):
    requests: Resource
    limits: Resource


class EnvVariable(BaseModel):
    name: str
    value: str


class Deployment(ProvisionerComponent):
    image: str
    resources: Resources
    pullSecret: List[str]
    env: List[EnvVariable]
    queueName: str
    maxReplicas: int


class CronTrigger(ProvisionerComponent):
    image: str
    resources: Resources
    pullSecret: List[str]
    env: List[EnvVariable]
    cron: str
    cmd: List[str]



"""
This is a generic trigger function that can be used
to trigger all specifications belonging to an app
"""

import os

from servc.svc.client.poll import pollMessage
from servc.svc.client.send import sendMessage
from servc.svc.com.bus.rabbitmq import BusRabbitMQ
from servc.svc.com.cache.redis import CacheRedis
from servc.svc.config import Config
from servc.svc.idgen.simple import simple
from servc.svc.io.input import InputPayload, InputType

ORCHESTRATOR_QUEUE = os.getenv("ORCHESTRATOR_QUEUE", "orchestrator")
PROVISIONER_QUEUE = os.getenv("PROVISIONER_QUEUE", "provisioner")
config = Config()
bus = BusRabbitMQ(config.get(f"conf.{BusRabbitMQ.name}"))
cache = CacheRedis(config.get(f"conf.{CacheRedis.name}"))


profile = {
    "profile_id": os.getenv("PROFILE_ID", ""),
    "app_id": os.getenv("APP_ID", ""),
}

# send a message to the provisioner to get all the tenants
tenantMessageId = sendMessage(
    {
        "id": "",
        "type": InputType.INPUT.value,
        "route": PROVISIONER_QUEUE,
        "force": True,
        "argumentId": "",
        "instanceId": "",
        "argument": {
            "method": "getspec",
            "inputs": profile,
        },
    },
    bus,
    cache,
    simple,
    force=True,
)
tenantSpecs = pollMessage(tenantMessageId, cache, timeout=5 * 60)
if tenantSpecs["isError"]:
    print("Error retrieving tenant specs:", tenantSpecs["responseBody"])
    exit(1)

for row in tenantSpecs["responseBody"]:
    tenant = row["auth_expression"]
    payload: InputPayload = {
        "id": "",
        "type": InputType.INPUT.value,
        "route": ORCHESTRATOR_QUEUE,
        "force": True,
        "argumentId": "",
        "instanceId": "",
        "argument": {
            "method": "trigger",
            "inputs": {
                **profile,
                "dag_version": None,
                "tenant_name": tenant,
                "payload": {},
            },
        },
    }

    id = sendMessage(payload, bus, cache, simple, force=True)
    trigger_response = pollMessage(id, cache, timeout=5 * 60)
    if trigger_response["isError"]:
        print(f"Error triggering tenant {tenant}:", trigger_response["responseBody"])
        exit(1)
    else:
        print(
            f"Successfully triggered tenant {tenant}:", trigger_response["responseBody"]
        )
